@@ -37,13 +37,98 @@ def statistics_report(data, outfile, context, analyte, parametric, pfdr, pi0_lam
 
     return(data)
 
+def infer_genes(infile, outfile, context, parametric, pfdr, pi0_lambda, pi0_method, pi0_smooth_df, pi0_smooth_log_pi0, lfdr_truncate, lfdr_monotone, lfdr_transformation, lfdr_adj, lfdr_eps):
+
+    con = sqlite3.connect(infile)
+
+    if not check_sqlite_table(con, "SCORE_MS2"):
+        raise click.ClickException("Apply scoring to MS2-level data before running gene-level scoring.")
+
+    if context in ['global','experiment-wide','run-specific']:
+        if context == 'global':
+            run_id = 'NULL'
+            group_id = 'GENE.ID'
+        else:
+            run_id = 'RUN_ID'
+            group_id = 'RUN_ID || "_" || GENE.ID'
+
+        con.executescript('''
+CREATE INDEX IF NOT EXISTS idx_peptide_gene_mapping_gene_id ON PEPTIDE_GENE_MAPPING (GENE_ID);
+CREATE INDEX IF NOT EXISTS idx_peptide_gene_mapping_peptide_id ON PEPTIDE_GENE_MAPPING (PEPTIDE_ID);
+CREATE INDEX IF NOT EXISTS idx_peptide_peptide_id ON PEPTIDE (ID);
+CREATE INDEX IF NOT EXISTS idx_precursor_peptide_mapping_peptide_id ON PRECURSOR_PEPTIDE_MAPPING (PEPTIDE_ID);
+CREATE INDEX IF NOT EXISTS idx_precursor_peptide_mapping_precursor_id ON PRECURSOR_PEPTIDE_MAPPING (PRECURSOR_ID);
+CREATE INDEX IF NOT EXISTS idx_precursor_precursor_id ON PRECURSOR (ID);
+CREATE INDEX IF NOT EXISTS idx_feature_precursor_id ON FEATURE (PRECURSOR_ID);
+CREATE INDEX IF NOT EXISTS idx_feature_feature_id ON FEATURE (ID);
+CREATE INDEX IF NOT EXISTS idx_score_ms2_feature_id ON SCORE_MS2 (FEATURE_ID);
+''')
+
+        data = pd.read_sql_query('''
+SELECT %s AS RUN_ID,
+       %s AS GROUP_ID,
+       GENE.ID AS GENE_ID,
+       PRECURSOR.DECOY AS DECOY,
+       SCORE,
+       "%s" AS CONTEXT
+FROM GENE
+INNER JOIN
+  (SELECT PEPTIDE_GENE_MAPPING.PEPTIDE_ID AS PEPTIDE_ID,
+          GENE_ID
+   FROM
+     (SELECT PEPTIDE_ID,
+             COUNT(*) AS NUM_GENES
+      FROM PEPTIDE_GENE_MAPPING
+      GROUP BY PEPTIDE_ID) AS GENES_PER_PEPTIDE
+   INNER JOIN PEPTIDE_GENE_MAPPING ON GENES_PER_PEPTIDE.PEPTIDE_ID = PEPTIDE_GENE_MAPPING.PEPTIDE_ID
+   WHERE NUM_GENES == 1) AS PEPTIDE_GENE_MAPPING ON GENE.ID = PEPTIDE_GENE_MAPPING.GENE_ID
+INNER JOIN PEPTIDE ON PEPTIDE_GENE_MAPPING.PEPTIDE_ID = PEPTIDE.ID
+INNER JOIN PRECURSOR_PEPTIDE_MAPPING ON PEPTIDE.ID = PRECURSOR_PEPTIDE_MAPPING.PEPTIDE_ID
+INNER JOIN PRECURSOR ON PRECURSOR_PEPTIDE_MAPPING.PRECURSOR_ID = PRECURSOR.ID
+INNER JOIN FEATURE ON PRECURSOR.ID = FEATURE.PRECURSOR_ID
+INNER JOIN SCORE_MS2 ON FEATURE.ID = SCORE_MS2.FEATURE_ID
+GROUP BY GROUP_ID
+HAVING MAX(SCORE)
+ORDER BY SCORE DESC
+''' % (run_id, group_id, context), con)
+    else:
+        raise click.ClickException("Unspecified context selected.")
+
+    data.columns = [col.lower() for col in data.columns]
+    con.close()
+
+    if context == 'run-specific':
+        data = data.groupby('run_id').apply(statistics_report, outfile, context, "gene", parametric, pfdr, pi0_lambda, pi0_method, pi0_smooth_df, pi0_smooth_log_pi0, lfdr_truncate, lfdr_monotone, lfdr_transformation, lfdr_adj, lfdr_eps).reset_index()
+
+    elif context in ['global', 'experiment-wide']:
+        data = statistics_report(data, outfile, context, "gene", parametric, pfdr, pi0_lambda, pi0_method, pi0_smooth_df, pi0_smooth_log_pi0, lfdr_truncate, lfdr_monotone, lfdr_transformation, lfdr_adj, lfdr_eps)
+
+    # store data in table
+    if infile != outfile:
+        copyfile(infile, outfile)
+
+    con = sqlite3.connect(outfile)
+
+    c = con.cursor()
+    c.execute('SELECT count(name) FROM sqlite_master WHERE type="table" AND name="SCORE_GENE"')
+    if c.fetchone()[0] == 1:
+        c.execute('DELETE FROM SCORE_GENE WHERE CONTEXT =="%s"' % context)
+    c.fetchall()
+
+    df = data[['context','run_id','gene_id','score','p_value','q_value','pep']]
+    df.columns = ['CONTEXT','RUN_ID','GENE_ID','SCORE','PVALUE','QVALUE','PEP']
+    table = "SCORE_GENE"
+    df.to_sql(table, con, index=False, dtype={"RUN_ID": "INTEGER"}, if_exists='append')
+
+    con.close()
+
 
 def infer_proteins(infile, outfile, context, parametric, pfdr, pi0_lambda, pi0_method, pi0_smooth_df, pi0_smooth_log_pi0, lfdr_truncate, lfdr_monotone, lfdr_transformation, lfdr_adj, lfdr_eps):
 
     con = sqlite3.connect(infile)
 
     if not check_sqlite_table(con, "SCORE_MS2"):
-        sys.exit("Error: Apply scoring to MS2-level data before running protein-level scoring.")
+        raise click.ClickException("Apply scoring to MS2-level data before running protein-level scoring.")
 
     if context in ['global','experiment-wide','run-specific']:
         if context == 'global':
@@ -93,7 +178,7 @@ HAVING MAX(SCORE)
 ORDER BY SCORE DESC
 ''' % (run_id, group_id, context), con)
     else:
-        sys.exit("Error: Unspecified context selected.")
+        raise click.ClickException("Unspecified context selected.")
 
     data.columns = [col.lower() for col in data.columns]
     con.close()
@@ -129,7 +214,7 @@ def infer_peptides(infile, outfile, context, parametric, pfdr, pi0_lambda, pi0_m
     con = sqlite3.connect(infile)
 
     if not check_sqlite_table(con, "SCORE_MS2"):
-        sys.exit("Error: Apply scoring to MS2-level data before running peptide-level scoring.")
+        raise click.ClickException("Apply scoring to MS2-level data before running peptide-level scoring.")
 
     if context in ['global','experiment-wide','run-specific']:
         if context == 'global':
@@ -166,7 +251,7 @@ HAVING MAX(SCORE)
 ORDER BY SCORE DESC
 ''' % (run_id, group_id, context), con)
     else:
-        sys.exit("Error: Unspecified context selected.")
+        raise click.ClickException("Unspecified context selected.")
 
     data.columns = [col.lower() for col in data.columns]
     con.close()
@@ -197,50 +282,41 @@ ORDER BY SCORE DESC
     con.close()
 
 
-def merge_osw(infiles, outfile, subsample_ratio, test):
-    # Copy the first file to have a template
-    copyfile(infiles[0], outfile)
+def subsample_osw(infile, outfile, subsample_ratio, test):
+    conn = sqlite3.connect(infile)
+    ms1_present = check_sqlite_table(conn, "FEATURE_MS1")
+    ms2_present = check_sqlite_table(conn, "FEATURE_MS2")
+    transition_present = check_sqlite_table(conn, "FEATURE_TRANSITION")
+    conn.close()
+
     conn = sqlite3.connect(outfile)
     c = conn.cursor()
+
     c.executescript('''
 PRAGMA synchronous = OFF;
 
-DELETE FROM RUN;
-
-DELETE FROM FEATURE;
-
-DELETE FROM FEATURE_MS1;
-
-DELETE FROM FEATURE_MS2;
-
-DELETE FROM FEATURE_TRANSITION;
-''')
-
-    for infile in infiles:
-        c.executescript('''
 ATTACH DATABASE "%s" AS sdb;
 
-INSERT INTO RUN SELECT * FROM sdb.RUN;
+CREATE TABLE RUN AS SELECT * FROM sdb.RUN;
 
 DETACH DATABASE sdb;
 ''' % infile)
-        click.echo("Info: Merged runs of file %s to %s." % (infile, outfile))
+    click.echo("Info: Propagated runs of file %s to %s." % (infile, outfile))
 
-    for infile in infiles:
-        if subsample_ratio >= 1.0:
-            c.executescript('''
+    if subsample_ratio >= 1.0:
+        c.executescript('''
 ATTACH DATABASE "%s" AS sdb; 
 
-INSERT INTO FEATURE SELECT * FROM sdb.FEATURE; 
+CREATE TABLE FEATURE AS SELECT * FROM sdb.FEATURE; 
 
 DETACH DATABASE sdb;
 ''' % infile)
-        else:
-            if test:
-                c.executescript('''
+    else:
+        if test:
+            c.executescript('''
 ATTACH DATABASE "%s" AS sdb;
 
-INSERT INTO FEATURE
+CREATE TABLE FEATURE AS 
 SELECT *
 FROM sdb.FEATURE
 WHERE PRECURSOR_ID IN
@@ -252,11 +328,11 @@ WHERE PRECURSOR_ID IN
 
 DETACH DATABASE sdb;
 ''' % (infile, subsample_ratio))
-            else:
-                c.executescript('''
+        else:
+            c.executescript('''
 ATTACH DATABASE "%s" AS sdb;
 
-INSERT INTO FEATURE
+CREATE TABLE FEATURE AS 
 SELECT *
 FROM sdb.FEATURE
 WHERE PRECURSOR_ID IN
@@ -269,14 +345,14 @@ WHERE PRECURSOR_ID IN
 
 DETACH DATABASE sdb;
 ''' % (infile, subsample_ratio))
-        click.echo("Info: Merged generic features of file %s to %s." % (infile, outfile))
+    click.echo("Info: Subsampled generic features of file %s to %s." % (infile, outfile))
 
-    for infile in infiles:
+    if ms1_present:
         if subsample_ratio >= 1.0:
             c.executescript('''
 ATTACH DATABASE "%s" AS sdb;
 
-INSERT INTO FEATURE_MS1
+CREATE TABLE FEATURE_MS1 AS 
 SELECT *
 FROM sdb.FEATURE_MS1;
 
@@ -286,7 +362,7 @@ DETACH DATABASE sdb;
             c.executescript('''
 ATTACH DATABASE "%s" AS sdb;
 
-INSERT INTO FEATURE_MS1
+CREATE TABLE FEATURE_MS1 AS 
 SELECT *
 FROM sdb.FEATURE_MS1
 WHERE sdb.FEATURE_MS1.FEATURE_ID IN
@@ -295,14 +371,14 @@ WHERE sdb.FEATURE_MS1.FEATURE_ID IN
 
 DETACH DATABASE sdb;
 ''' % infile)
-        click.echo("Info: Merged MS1 features of file %s to %s." % (infile, outfile))
+        click.echo("Info: Subsampled MS1 features of file %s to %s." % (infile, outfile))
 
-    for infile in infiles:
+    if ms2_present:
         if subsample_ratio >= 1.0:
             c.executescript('''
 ATTACH DATABASE "%s" AS sdb;
 
-INSERT INTO FEATURE_MS2
+CREATE TABLE FEATURE_MS2 AS 
 SELECT *
 FROM sdb.FEATURE_MS2;
 
@@ -312,7 +388,7 @@ DETACH DATABASE sdb;
             c.executescript('''
 ATTACH DATABASE "%s" AS sdb;
 
-INSERT INTO FEATURE_MS2
+CREATE TABLE FEATURE_MS2 AS 
 SELECT *
 FROM sdb.FEATURE_MS2
 WHERE sdb.FEATURE_MS2.FEATURE_ID IN
@@ -321,14 +397,14 @@ WHERE sdb.FEATURE_MS2.FEATURE_ID IN
 
 DETACH DATABASE sdb;
 ''' % infile)
-        click.echo("Info: Merged MS2 features of file %s to %s." % (infile, outfile))
+        click.echo("Info: Subsampled MS2 features of file %s to %s." % (infile, outfile))
 
-    for infile in infiles:
+    if transition_present:
         if subsample_ratio >= 1.0:
             c.executescript('''
 ATTACH DATABASE "%s" AS sdb;
 
-INSERT INTO FEATURE_TRANSITION
+CREATE TABLE FEATURE_TRANSITION AS 
 SELECT *
 FROM sdb.FEATURE_TRANSITION;
 
@@ -338,7 +414,7 @@ DETACH DATABASE sdb;
             c.executescript('''
 ATTACH DATABASE "%s" AS sdb;
 
-INSERT INTO FEATURE_TRANSITION
+CREATE TABLE FEATURE_TRANSITION AS 
 SELECT *
 FROM sdb.FEATURE_TRANSITION
 WHERE sdb.FEATURE_TRANSITION.FEATURE_ID IN
@@ -347,18 +423,18 @@ WHERE sdb.FEATURE_TRANSITION.FEATURE_ID IN
 
 DETACH DATABASE sdb;
 ''' % infile)
-        click.echo("Info: Merged transition features of file %s to %s." % (infile, outfile))
+        click.echo("Info: Subsampled transition features of file %s to %s." % (infile, outfile))
 
     conn.commit()
     conn.close()
 
-    click.echo("Info: All OSW files were merged.")
+    click.echo("Info: OSW file was subsampled.")
 
 
 def reduce_osw(infile, outfile):
     conn = sqlite3.connect(infile)
     if not check_sqlite_table(conn, "SCORE_MS2"):
-        sys.exit("Error: Apply scoring to MS2 data before reducing file for multi-run scoring.")
+        raise click.ClickException("Apply scoring to MS2 data before reducing file for multi-run scoring.")
     conn.close()
 
     try:
@@ -409,11 +485,195 @@ WHERE ID IN
     click.echo("Info: OSW file was reduced for multi-run scoring.")
 
 
-def merge_oswr(infiles, outfile, templatefile):
+def merge_osw(infiles, outfile, templatefile, same_run):
+    conn = sqlite3.connect(infiles[0])
+    reduced = check_sqlite_table(conn, "SCORE_MS2")
+    conn.close()
+
+    if reduced:
+        merge_oswr(infiles, outfile, templatefile, same_run)
+    else:
+        merge_osws(infiles, outfile, templatefile, same_run)
+
+
+def merge_osws(infiles, outfile, templatefile, same_run):
+    # Copy the first file to have a template
+    copyfile(templatefile, outfile)
+    conn = sqlite3.connect(outfile)
+    c = conn.cursor()
+    if same_run:
+        c.execute("SELECT ID, FILENAME FROM RUN")
+        result = c.fetchall()
+        if len(result) != 1:
+            raise click.ClickException("Input for same-run merge contains more than one run.")
+        runid, rname = result[0]
+
+    c.executescript('''
+PRAGMA synchronous = OFF;
+
+DROP TABLE IF EXISTS RUN;
+
+DROP TABLE IF EXISTS FEATURE;
+
+DROP TABLE IF EXISTS FEATURE_MS1;
+
+DROP TABLE IF EXISTS FEATURE_MS2;
+
+DROP TABLE IF EXISTS FEATURE_TRANSITION;
+
+DROP TABLE IF EXISTS SCORE_MS1;
+
+DROP TABLE IF EXISTS SCORE_MS2;
+
+DROP TABLE IF EXISTS SCORE_TRANSITION;
+
+DROP TABLE IF EXISTS SCORE_PEPTIDE;
+
+DROP TABLE IF EXISTS SCORE_PROTEIN;
+
+DROP TABLE IF EXISTS SCORE_IPF;
+
+ATTACH DATABASE "%s" AS sdb;
+
+CREATE TABLE RUN AS SELECT * FROM sdb.RUN LIMIT 0;
+
+CREATE TABLE FEATURE AS SELECT * FROM sdb.FEATURE LIMIT 0;
+
+CREATE TABLE FEATURE_MS1 AS SELECT * FROM sdb.FEATURE_MS1 LIMIT 0;
+
+CREATE TABLE FEATURE_MS2 AS SELECT * FROM sdb.FEATURE_MS2 LIMIT 0;
+
+CREATE TABLE FEATURE_TRANSITION AS SELECT * FROM sdb.FEATURE_TRANSITION LIMIT 0;
+
+DETACH DATABASE sdb;
+''' % (infiles[0]))
+
+    conn.commit()
+    conn.close()
+
+    for infile in infiles:
+        conn = sqlite3.connect(outfile)
+        c = conn.cursor()
+
+        # Only create a single run entry (all files are presumably from the same run)
+        if same_run:
+            c.executescript('''INSERT INTO RUN (ID, FILENAME) VALUES (%s, '%s')''' % (runid, rname) )
+            break;
+        else:
+            c.executescript('''
+ATTACH DATABASE "%s" AS sdb;
+
+INSERT INTO RUN SELECT * FROM sdb.RUN;
+
+DETACH DATABASE sdb;
+''' % infile)
+
+        conn.commit()
+        conn.close()
+
+        click.echo("Info: Merged runs of file %s to %s." % (infile, outfile))
+
+    # Now merge the run-specific data into the output file:
+    #   Note: only tables FEATURE, FEATURE_MS1, FEATURE_MS2 and FEATURE_TRANSITION are run-specific
+    for infile in infiles:
+        conn = sqlite3.connect(outfile)
+        c = conn.cursor()
+
+        c.executescript('''
+ATTACH DATABASE "%s" AS sdb; 
+
+INSERT INTO FEATURE SELECT * FROM sdb.FEATURE; 
+
+DETACH DATABASE sdb;
+''' % infile)
+
+        conn.commit()
+        conn.close()
+
+        click.echo("Info: Merged generic features of file %s to %s." % (infile, outfile))
+        
+    if same_run:
+        conn = sqlite3.connect(outfile)
+        c = conn.cursor()
+
+        # Fix run id assuming we only have a single run
+        c.executescript('''UPDATE FEATURE SET RUN_ID = %s''' % runid)
+
+        conn.commit()
+        conn.close()
+
+    for infile in infiles:
+        conn = sqlite3.connect(outfile)
+        c = conn.cursor()
+
+        c.executescript('''
+ATTACH DATABASE "%s" AS sdb;
+
+INSERT INTO FEATURE_MS1
+SELECT *
+FROM sdb.FEATURE_MS1;
+
+DETACH DATABASE sdb;
+''' % infile)
+
+        conn.commit()
+        conn.close()
+
+        click.echo("Info: Merged MS1 features of file %s to %s." % (infile, outfile))
+
+    for infile in infiles:
+        conn = sqlite3.connect(outfile)
+        c = conn.cursor()
+
+        c.executescript('''
+ATTACH DATABASE "%s" AS sdb;
+
+INSERT INTO FEATURE_MS2
+SELECT *
+FROM sdb.FEATURE_MS2;
+
+DETACH DATABASE sdb;
+''' % infile)
+
+        conn.commit()
+        conn.close()
+
+        click.echo("Info: Merged MS2 features of file %s to %s." % (infile, outfile))
+
+    for infile in infiles:
+        conn = sqlite3.connect(outfile)
+        c = conn.cursor()
+
+        c.executescript('''
+ATTACH DATABASE "%s" AS sdb;
+
+INSERT INTO FEATURE_TRANSITION
+SELECT *
+FROM sdb.FEATURE_TRANSITION;
+
+DETACH DATABASE sdb;
+''' % infile)
+
+        conn.commit()
+        conn.close()
+
+        click.echo("Info: Merged transition features of file %s to %s." % (infile, outfile))
+
+    click.echo("Info: All OSWS files were merged.")
+
+
+def merge_oswr(infiles, outfile, templatefile, same_run):
     # Copy the template to the output file
     copyfile(templatefile, outfile)
     conn = sqlite3.connect(outfile)
     c = conn.cursor()
+    if same_run:
+        c.execute("SELECT ID, FILENAME FROM RUN")
+        result = c.fetchall()
+        if len(result) != 1:
+            raise click.ClickException("Input for same-run merge contains more than one run.")
+        runid, rname = result[0]
+
     c.executescript('''
 PRAGMA synchronous = OFF;
 
@@ -449,20 +709,56 @@ CREATE TABLE FEATURE(ID INT PRIMARY KEY NOT NULL,
                      PRECURSOR_ID INT NOT NULL);
 ''')
 
+    conn.commit()
+    conn.close()
+
     for infile in infiles:
-        c.executescript('ATTACH DATABASE "%s" AS sdb; INSERT INTO RUN SELECT * FROM sdb.RUN; DETACH DATABASE sdb;' % infile)
+        conn = sqlite3.connect(outfile)
+        c = conn.cursor()
+
+        # Only create a single run entry (all files are presumably from the same run)
+        if same_run:
+            c.executescript('''INSERT INTO RUN (ID, FILENAME) VALUES (%s, '%s')''' % (runid, rname) )
+            break;
+        else:
+            c.executescript('ATTACH DATABASE "%s" AS sdb; INSERT INTO RUN SELECT * FROM sdb.RUN; DETACH DATABASE sdb;' % infile)
+
+        conn.commit()
+        conn.close()
+
         click.echo("Info: Merged runs of file %s to %s." % (infile, outfile))
 
     for infile in infiles:
+        conn = sqlite3.connect(outfile)
+        c = conn.cursor()
+
         c.executescript('ATTACH DATABASE "%s" AS sdb; INSERT INTO FEATURE SELECT * FROM sdb.FEATURE; DETACH DATABASE sdb;' % infile)
+
+        conn.commit()
+        conn.close()
+
         click.echo("Info: Merged generic features of file %s to %s." % (infile, outfile))
 
-    for infile in infiles:
-        c.executescript('ATTACH DATABASE "%s" AS sdb; INSERT INTO SCORE_MS2 SELECT * FROM sdb.SCORE_MS2; DETACH DATABASE sdb;' % infile)
-        click.echo("Info: Merged MS2 scores of file %s to %s." % (infile, outfile))
+    if same_run:
+        conn = sqlite3.connect(outfile)
+        c = conn.cursor()
 
-    conn.commit()
-    conn.close()
+        # Fix run id assuming we only have a single run
+        c.executescript('''UPDATE FEATURE SET RUN_ID = %s''' % runid)
+
+        conn.commit()
+        conn.close()
+
+    for infile in infiles:
+        conn = sqlite3.connect(outfile)
+        c = conn.cursor()
+
+        c.executescript('ATTACH DATABASE "%s" AS sdb; INSERT INTO SCORE_MS2 SELECT * FROM sdb.SCORE_MS2; DETACH DATABASE sdb;' % infile)
+
+        conn.commit()
+        conn.close()
+
+        click.echo("Info: Merged MS2 scores of file %s to %s." % (infile, outfile))
 
     click.echo("Info: All reduced OSWR files were merged.")
 
@@ -472,30 +768,38 @@ def backpropagate_oswr(infile, outfile, apply_scores):
     if infile != outfile:
         copyfile(infile, outfile)
 
+    # find out what tables exist in the scores
+    score_con = sqlite3.connect(apply_scores)
+    peptide_present = check_sqlite_table(score_con, "SCORE_PEPTIDE")
+    protein_present = check_sqlite_table(score_con, "SCORE_PROTEIN")
+    score_con.close()
+    if not (peptide_present or protein_present):
+        raise click.ClickException('Backpropagation requires peptide or protein-level contexts.')
+
+    # build up the list
+    script = list()
+    script.append('PRAGMA synchronous = OFF;')
+    script.append('DROP TABLE IF EXISTS SCORE_PEPTIDE;')
+    script.append('DROP TABLE IF EXISTS SCORE_PROTEIN;')
+
+    # create the tables
+    if peptide_present:
+        script.append('CREATE TABLE SCORE_PEPTIDE (CONTEXT TEXT, RUN_ID INTEGER, PEPTIDE_ID INTEGER, SCORE REAL, PVALUE REAL, QVALUE REAL, PEP REAL);')
+    if protein_present:
+        script.append('CREATE TABLE SCORE_PROTEIN (CONTEXT TEXT, RUN_ID INTEGER, PROTEIN_ID INTEGER, SCORE REAL, PVALUE REAL, QVALUE REAL, PEP REAL);')
+
+    # copy across the tables
+    script.append('ATTACH DATABASE "{}" AS sdb;'.format(apply_scores))
+    insert_table_fmt = 'INSERT INTO {0}\nSELECT *\nFROM sdb.{0};'
+    if peptide_present:
+        script.append(insert_table_fmt.format('SCORE_PEPTIDE'))
+    if protein_present:
+        script.append(insert_table_fmt.format('SCORE_PROTEIN'))
+
+    # execute the script
     conn = sqlite3.connect(outfile)
     c = conn.cursor()
-    c.executescript('''
-PRAGMA synchronous = OFF;
-
-DROP TABLE IF EXISTS SCORE_PEPTIDE;
-
-DROP TABLE IF EXISTS SCORE_PROTEIN;
-
-CREATE TABLE SCORE_PEPTIDE (CONTEXT TEXT, RUN_ID INTEGER, PEPTIDE_ID INTEGER, SCORE REAL, PVALUE REAL, QVALUE REAL, PEP REAL);
-
-CREATE TABLE SCORE_PROTEIN (CONTEXT TEXT, RUN_ID INTEGER, PROTEIN_ID INTEGER, SCORE REAL, PVALUE REAL, QVALUE REAL, PEP REAL);
-
-ATTACH DATABASE "%s" AS sdb;
-
-INSERT INTO SCORE_PEPTIDE
-SELECT *
-FROM sdb.SCORE_PEPTIDE;
-
-INSERT INTO SCORE_PROTEIN
-SELECT *
-FROM sdb.SCORE_PROTEIN;
-''' % apply_scores)
-
+    c.executescript('\n'.join(script))
     conn.commit()
     conn.close()
 
