@@ -5,11 +5,16 @@ import sys
 
 from .runner import PyProphetLearner, PyProphetWeightApplier
 from .ipf import infer_peptidoforms
-from .levels_contexts import infer_peptides, infer_proteins, merge_osw, reduce_osw, merge_oswr, backpropagate_oswr
+from .levels_contexts import infer_peptides, infer_proteins, infer_genes, subsample_osw, reduce_osw, merge_osw, backpropagate_oswr
 from .export import export_tsv, export_score_plots
+from .export_compound import export_compound_tsv
 from .filter import filter_sqmass
-from .data_handling import (transform_pi0_lambda, transform_threads, transform_subsample_ratio)
+from .data_handling import (transform_pi0_lambda, transform_threads, transform_subsample_ratio, check_sqlite_table)
 from functools import update_wrapper
+import sqlite3
+from tabulate import tabulate
+
+from hyperopt import hp
 
 try:
     profile
@@ -33,6 +38,9 @@ def cli():
 @click.option('--in', 'infile', required=True, type=click.Path(exists=True), help='PyProphet input file.')
 @click.option('--out', 'outfile', type=click.Path(exists=False), help='PyProphet output file.')
 # Semi-supervised learning
+@click.option('--classifier', default='LDA', show_default=True, type=click.Choice(['LDA', 'XGBoost']), help='Either a "LDA" or "XGBoost" classifier is used for semi-supervised learning.')
+@click.option('--xgb_autotune/--no-xgb_autotune', default=False, show_default=True, help='XGBoost: Autotune hyperparameters.')
+
 @click.option('--apply_weights', type=click.Path(exists=True), help='Apply PyProphet score weights file instead of semi-supervised learning.')
 @click.option('--xeval_fraction', default=0.5, show_default=True, type=float, help='Data fraction used for cross-validation of semi-supervised learning step.')
 @click.option('--xeval_num_iter', default=10, show_default=True, type=int, help='Number of iterations for cross-validation of semi-supervised learning step.')
@@ -65,7 +73,7 @@ def cli():
 # Processing
 @click.option('--threads', default=1, show_default=True, type=int, help='Number of threads used for semi-supervised learning. -1 means all available CPUs.', callback=transform_threads)
 @click.option('--test/--no-test', default=False, show_default=True, help='Run in test mode with fixed seed.')
-def score(infile, outfile, apply_weights, xeval_fraction, xeval_num_iter, ss_initial_fdr, ss_iteration_fdr, ss_num_iter, ss_main_score, group_id, parametric, pfdr, pi0_lambda, pi0_method, pi0_smooth_df, pi0_smooth_log_pi0, lfdr_truncate, lfdr_monotone, lfdr_transformation, lfdr_adj, lfdr_eps, level, ipf_max_peakgroup_rank, ipf_max_peakgroup_pep, ipf_max_transition_isotope_overlap, ipf_min_transition_sn, tric_chromprob, threads, test):
+def score(infile, outfile, classifier, xgb_autotune, apply_weights, xeval_fraction, xeval_num_iter, ss_initial_fdr, ss_iteration_fdr, ss_num_iter, ss_main_score, group_id, parametric, pfdr, pi0_lambda, pi0_method, pi0_smooth_df, pi0_smooth_log_pi0, lfdr_truncate, lfdr_monotone, lfdr_transformation, lfdr_adj, lfdr_eps, level, ipf_max_peakgroup_rank, ipf_max_peakgroup_pep, ipf_max_transition_isotope_overlap, ipf_min_transition_sn, tric_chromprob, threads, test):
     """
     Conduct semi-supervised learning and error-rate estimation for MS1, MS2 and transition-level data. 
     """
@@ -75,10 +83,17 @@ def score(infile, outfile, apply_weights, xeval_fraction, xeval_num_iter, ss_ini
     else:
         outfile = outfile
 
+    # Prepare XGBoost-specific parameters
+    xgb_hyperparams = {'autotune': xgb_autotune, 'autotune_num_rounds': 10, 'num_boost_round': 100, 'early_stopping_rounds': 10, 'test_size': 0.33}
+
+    xgb_params = {'eta': 0.3, 'gamma': 0, 'max_depth': 6, 'min_child_weight': 1, 'subsample': 1, 'colsample_bytree': 1, 'colsample_bylevel': 1, 'colsample_bynode': 1, 'lambda': 1, 'alpha': 0, 'scale_pos_weight': 1, 'silent': 1, 'objective': 'binary:logitraw', 'nthread': 1, 'eval_metric': 'auc'}
+
+    xgb_params_space = {'eta': hp.uniform('eta', 0.0, 0.3), 'gamma': hp.uniform('gamma', 0.0, 0.5), 'max_depth': hp.quniform('max_depth', 2, 8, 1), 'min_child_weight': hp.quniform('min_child_weight', 1, 5, 1), 'subsample': 1, 'colsample_bytree': 1, 'colsample_bylevel': 1, 'colsample_bynode': 1, 'lambda': hp.uniform('lambda', 0.0, 1.0), 'alpha': hp.uniform('alpha', 0.0, 1.0), 'scale_pos_weight': 1.0, 'silent': 1, 'objective': 'binary:logitraw', 'nthread': 1, 'eval_metric': 'auc'}
+
     if not apply_weights:
-        PyProphetLearner(infile, outfile, xeval_fraction, xeval_num_iter, ss_initial_fdr, ss_iteration_fdr, ss_num_iter, ss_main_score, group_id, parametric, pfdr, pi0_lambda, pi0_method, pi0_smooth_df, pi0_smooth_log_pi0, lfdr_truncate, lfdr_monotone, lfdr_transformation, lfdr_adj, lfdr_eps, level, ipf_max_peakgroup_rank, ipf_max_peakgroup_pep, ipf_max_transition_isotope_overlap, ipf_min_transition_sn, tric_chromprob, threads, test).run()
+        PyProphetLearner(infile, outfile, classifier, xgb_hyperparams, xgb_params, xgb_params_space, xeval_fraction, xeval_num_iter, ss_initial_fdr, ss_iteration_fdr, ss_num_iter, ss_main_score, group_id, parametric, pfdr, pi0_lambda, pi0_method, pi0_smooth_df, pi0_smooth_log_pi0, lfdr_truncate, lfdr_monotone, lfdr_transformation, lfdr_adj, lfdr_eps, level, ipf_max_peakgroup_rank, ipf_max_peakgroup_pep, ipf_max_transition_isotope_overlap, ipf_min_transition_sn, tric_chromprob, threads, test).run()
     else:
-        PyProphetWeightApplier(infile, outfile, xeval_fraction, xeval_num_iter, ss_initial_fdr, ss_iteration_fdr, ss_num_iter, ss_main_score, group_id, parametric, pfdr, pi0_lambda, pi0_method, pi0_smooth_df, pi0_smooth_log_pi0, lfdr_truncate, lfdr_monotone, lfdr_transformation, lfdr_adj, lfdr_eps, level, ipf_max_peakgroup_rank, ipf_max_peakgroup_pep, ipf_max_transition_isotope_overlap, ipf_min_transition_sn, tric_chromprob, threads, test, apply_weights).run()
+        PyProphetWeightApplier(infile, outfile, classifier, xgb_hyperparams, xgb_params, xgb_params_space, xeval_fraction, xeval_num_iter, ss_initial_fdr, ss_iteration_fdr, ss_num_iter, ss_main_score, group_id, parametric, pfdr, pi0_lambda, pi0_method, pi0_smooth_df, pi0_smooth_log_pi0, lfdr_truncate, lfdr_monotone, lfdr_transformation, lfdr_adj, lfdr_eps, level, ipf_max_peakgroup_rank, ipf_max_peakgroup_pep, ipf_max_transition_isotope_overlap, ipf_min_transition_sn, tric_chromprob, threads, test, apply_weights).run()
 
 
 # IPF
@@ -140,6 +155,37 @@ def peptide(infile, outfile, context, parametric, pfdr, pi0_lambda, pi0_method,
     infer_peptides(infile, outfile, context, parametric, pfdr, pi0_lambda, pi0_method, pi0_smooth_df, pi0_smooth_log_pi0, lfdr_truncate, lfdr_monotone, lfdr_transformation, lfdr_adj, lfdr_eps)
 
 
+# Gene-level inference
+@cli.command()
+# File handling
+@click.option('--in', 'infile', required=True, type=click.Path(exists=True), help='PyProphet input file.')
+@click.option('--out', 'outfile', type=click.Path(exists=False), help='PyProphet output file.')
+# Context
+@click.option('--context', default='run-specific', show_default=True, type=click.Choice(['run-specific', 'experiment-wide', 'global']), help='Context to estimate gene-level FDR control.')
+# Statistics
+@click.option('--parametric/--no-parametric', default=False, show_default=True, help='Do parametric estimation of p-values.')
+@click.option('--pfdr/--no-pfdr', default=False, show_default=True, help='Compute positive false discovery rate (pFDR) instead of FDR.')
+@click.option('--pi0_lambda', default=[0.1,0.5,0.05], show_default=True, type=(float, float, float), help='Use non-parametric estimation of p-values. Either use <START END STEPS>, e.g. 0.1, 1.0, 0.1 or set to fixed value, e.g. 0.4, 0, 0.', callback=transform_pi0_lambda)
+@click.option('--pi0_method', default='bootstrap', show_default=True, type=click.Choice(['smoother', 'bootstrap']), help='Either "smoother" or "bootstrap"; the method for automatically choosing tuning parameter in the estimation of pi_0, the proportion of true null hypotheses.')
+@click.option('--pi0_smooth_df', default=3, show_default=True, type=int, help='Number of degrees-of-freedom to use when estimating pi_0 with a smoother.')
+@click.option('--pi0_smooth_log_pi0/--no-pi0_smooth_log_pi0', default=False, show_default=True, help='If True and pi0_method = "smoother", pi0 will be estimated by applying a smoother to a scatterplot of log(pi0) estimates against the tuning parameter lambda.')
+@click.option('--lfdr_truncate/--no-lfdr_truncate', show_default=True, default=True, help='If True, local FDR values >1 are set to 1.')
+@click.option('--lfdr_monotone/--no-lfdr_monotone', show_default=True, default=True, help='If True, local FDR values are non-decreasing with increasing p-values.')
+@click.option('--lfdr_transformation', default='probit', show_default=True, type=click.Choice(['probit', 'logit']), help='Either a "probit" or "logit" transformation is applied to the p-values so that a local FDR estimate can be formed that does not involve edge effects of the [0,1] interval in which the p-values lie.')
+@click.option('--lfdr_adj', default=1.5, show_default=True, type=float, help='Numeric value that is applied as a multiple of the smoothing bandwidth used in the density estimation.')
+@click.option('--lfdr_eps', default=np.power(10.0,-8), show_default=True, type=float, help='Numeric value that is threshold for the tails of the empirical p-value distribution.')
+def gene(infile, outfile, context, parametric, pfdr, pi0_lambda, pi0_method, pi0_smooth_df, pi0_smooth_log_pi0, lfdr_truncate, lfdr_monotone, lfdr_transformation, lfdr_adj, lfdr_eps):
+    """
+    Infer genes and conduct error-rate estimation in different contexts.
+    """
+
+    if outfile is None:
+        outfile = infile
+    else:
+        outfile = outfile
+
+    infer_genes(infile, outfile, context, parametric, pfdr, pi0_lambda, pi0_method, pi0_smooth_df, pi0_smooth_log_pi0, lfdr_truncate, lfdr_monotone, lfdr_transformation, lfdr_adj, lfdr_eps)
+
 # Protein-level inference
 @cli.command()
 # File handling
@@ -172,25 +218,23 @@ def protein(infile, outfile, context, parametric, pfdr, pi0_lambda, pi0_method,
     infer_proteins(infile, outfile, context, parametric, pfdr, pi0_lambda, pi0_method, pi0_smooth_df, pi0_smooth_log_pi0, lfdr_truncate, lfdr_monotone, lfdr_transformation, lfdr_adj, lfdr_eps)
 
 
-# Merging & subsampling of multiple runs
+# Subsample OpenSWATH file to minimum for integrated scoring
 @cli.command()
-@click.argument('infiles', nargs=-1, type=click.Path(exists=True))
-@click.option('--out','outfile', type=click.Path(exists=False), help='Merged OSW output file.')
-@click.option('--template','templatefile', type=click.Path(exists=False), help='[Reduced OSWR only] Template PQP or OSW file.')
+@click.option('--in', 'infile', required=True, type=click.Path(exists=True), help='OpenSWATH input file.')
+@click.option('--out','outfile', type=click.Path(exists=False), help='Subsampled OSWS output file.')
 @click.option('--subsample_ratio', default=1, show_default=True, type=float, help='Subsample ratio used per input file.', callback=transform_subsample_ratio)
 @click.option('--test/--no-test', default=False, show_default=True, help='Run in test mode with fixed seed.')
-def merge(infiles, outfile, templatefile, subsample_ratio, test):
+def subsample(infile, outfile, subsample_ratio, test):
     """
-    Merge multiple OSW files and optionally subsample the data for faster learning.
+    Subsample OpenSWATH file to minimum for integrated scoring
     """
 
-    if len(infiles) < 1:
-        sys.exit("Error: At least one PyProphet input file needs to be defined.")
-
-    if templatefile is None:
-        merge_osw(infiles, outfile, subsample_ratio, test)
+    if outfile is None:
+        outfile = infile
     else:
-        merge_oswr(infiles, outfile, templatefile)
+        outfile = outfile
+
+    subsample_osw(infile, outfile, subsample_ratio, test)
 
 
 # Reduce scored PyProphet file to minimum for global scoring
@@ -210,6 +254,23 @@ def reduce(infile, outfile):
     reduce_osw(infile, outfile)
 
 
+# Merging of multiple runs
+@cli.command()
+@click.argument('infiles', nargs=-1, type=click.Path(exists=True))
+@click.option('--out','outfile', required=True, type=click.Path(exists=False), help='Merged OSW output file.')
+@click.option('--same_run/--no-same_run', default=False, help='Assume input files are from same run (deletes run information).')
+@click.option('--template','templatefile', required=True, type=click.Path(exists=False), help='Template OSW file.')
+def merge(infiles, outfile, same_run, templatefile):
+    """
+    Merge multiple OSW files and (for large experiments, it is recommended to subsample first).
+    """
+
+    if len(infiles) < 1:
+        raise click.ClickException("At least one PyProphet input file needs to be provided.")
+
+    merge_osw(infiles, outfile, templatefile, same_run)
+
+
 # Backpropagate multi-run peptide and protein scores to single files
 @cli.command()
 @click.option('--in', 'infile', required=True, type=click.Path(exists=True), help='Single run PyProphet input file.')
@@ -263,6 +324,32 @@ def export(infile, outfile, format, outcsv, transition_quantification, max_trans
         export_tsv(infile, outfile, format, outcsv, transition_quantification, max_transition_pep, ipf, ipf_max_peptidoform_pep, max_rs_peakgroup_qvalue, peptide, max_global_peptide_qvalue, protein, max_global_protein_qvalue)
 
 
+# Export Compound TSV
+@cli.command()
+#File handling
+@click.option('--in', 'infile', required=True, type=click.Path(exists=True), help='PyProphet input file.')
+@click.option('--out', 'outfile', type=click.Path(exists=False), help='Output TSV/CSV (matrix, legacy_merged) file.')
+@click.option('--format', default='legacy_merged', show_default=True, type=click.Choice(['matrix', 'legacy_merged','score_plots']), help='Export format, either matrix, legacy_merged (PyProphet) or score_plots format.')
+@click.option('--csv/--no-csv', 'outcsv', default=False, show_default=True, help='Export CSV instead of TSV file.')
+# Context
+@click.option('--max_rs_peakgroup_qvalue', default=0.05, show_default=True, type=float, help='[format: matrix/legacy] Filter results to maximum run-specific peak group-level q-value.')
+def export_compound(infile, outfile, format, outcsv, max_rs_peakgroup_qvalue):
+    """
+    Export Compound TSV/CSV tables
+    """
+    if format == "score_plots":
+        export_score_plots(infile)
+    else:
+        if outfile is None:
+            if outcsv:
+                outfile = infile.split(".osw")[0] + ".csv"
+            else:
+                outfile = infile.split(".osw")[0] + ".tsv"
+        else:
+            outfile = outfile
+
+        export_compound_tsv(infile, outfile, format, outcsv, max_rs_peakgroup_qvalue)
+
 # Filter sqMass files
 @cli.command()
 # File handling
@@ -277,3 +364,40 @@ def filter(sqmassfiles, infile, max_precursor_pep, max_peakgroup_pep, max_transi
     """
 
     filter_sqmass(sqmassfiles, infile, max_precursor_pep, max_peakgroup_pep, max_transition_pep)
+
+# Print statistics
+@cli.command()
+@click.option('--in', 'infile', required=True, type=click.Path(exists=True), help='PyProphet input file.')
+def statistics(infile):
+    """
+    Print PyProphet statistics
+    """
+
+    con = sqlite3.connect(infile)
+
+    qts = [0.01, 0.05, 0.10]
+
+    for qt in qts:
+        if check_sqlite_table(con, 'SCORE_MS2'):
+            peakgroups = pd.read_sql('SELECT * FROM SCORE_MS2 INNER JOIN FEATURE ON SCORE_MS2.feature_id = FEATURE.id INNER JOIN RUN ON FEATURE.RUN_ID = RUN.ID WHERE RANK==1;' , con)
+
+            click.echo("Total peakgroups (q-value<%s): %s" % (qt, len(peakgroups[peakgroups['QVALUE']<qt][['FEATURE_ID']].drop_duplicates())))
+            click.echo("Total peakgroups per run (q-value<%s):" % qt)
+            click.echo(tabulate(peakgroups[peakgroups['QVALUE']<qt].groupby(['FILENAME'])['FEATURE_ID'].nunique().reset_index(), showindex=False))
+            click.echo(10*"=")
+
+        if check_sqlite_table(con, 'SCORE_PEPTIDE'):
+            peptides_global = pd.read_sql('SELECT * FROM SCORE_PEPTIDE WHERE CONTEXT=="global";' , con)
+            peptides = pd.read_sql('SELECT * FROM SCORE_PEPTIDE INNER JOIN RUN ON SCORE_PEPTIDE.RUN_ID = RUN.ID;' , con)
+
+            click.echo("Total peptides (global context) (q-value<%s): %s" % (qt, len(peptides_global[peptides_global['QVALUE']<qt][['PEPTIDE_ID']].drop_duplicates())))
+            click.echo(tabulate(peptides[peptides['QVALUE']<qt].groupby(['FILENAME'])['PEPTIDE_ID'].nunique().reset_index(), showindex=False))
+            click.echo(10*"=")
+
+        if check_sqlite_table(con, 'SCORE_PROTEIN'):
+            proteins_global = pd.read_sql('SELECT * FROM SCORE_PROTEIN WHERE CONTEXT=="global";' , con)
+            proteins = pd.read_sql('SELECT * FROM SCORE_PROTEIN INNER JOIN RUN ON SCORE_PROTEIN.RUN_ID = RUN.ID;' , con)
+
+            click.echo("Total proteins (global context) (q-value<%s): %s" % (qt, len(proteins_global[proteins_global['QVALUE']<qt][['PROTEIN_ID']].drop_duplicates())))
+            click.echo(tabulate(proteins[proteins['QVALUE']<qt].groupby(['FILENAME'])['PROTEIN_ID'].nunique().reset_index(), showindex=False))
+            click.echo(10*"=")
